"""Entry point for: python -m meeting_helper.flet_gui"""

# Fix SSL certificate verification for Flet client download
import os
try:
    import certifi
    os.environ.setdefault("SSL_CERT_FILE", certifi.where())
except ImportError:
    pass

from meeting_helper.config import get_config
from meeting_helper.flet_gui.app import run_flet_gui

run_flet_gui(get_config())
